var fs = require('fs');
var ichabod = require('./index');
var config = require('./config');

var opts = require('optimist')
        .usage('Usage: $0 -m <serve|test> -c <config-file>')
        .alias('i', 'init').describe('i', 'Create a new config file at the specified path.')
        .alias('m', 'mode').describe('m', 'Mode to run in: serve or test.').default('m', 'test')
        .alias('c', 'config').describe('c', 'Path to an ichabod config.')
        .alias('r', 'runner').describe('r', 'Runner to execute the specs. [zombie, selenium]')
        .alias('b', 'browser').describe('b', 'Browser to use if using the selenium runner.')
        .alias('p', 'port').describe('p', 'Port to run the static server on.').default('p', 8000)
        .alias('h', 'help').describe('h', 'Show this message.'),
    argv = opts.argv;

if (argv.h) {
    console.log(opts.help());
    process.exit();
}

if (argv.i) {
    var json = config.createDefault() + '\n';

    if (argv.i.length > 0) {
        fs.writeFileSync(argv.i, json, 'utf-8');
    } else {
        console.log(json);
    }

    process.exit();
}

var command = argv.m,
    conf = config.createConfig(argv);

var runner = new ichabod.createRunner(conf);

if (!runner) {
    console.log("Invalid runner type.");
    process.exit(1);
}

// Start up the static server

var server = ichabod.startServer(conf);

console.log("Server started on => " + conf.staticUrl + "specrunner.html");

if (command === 'test') {
    // Execute all the tests and shut down the server at the end
    runner.start(function() {
        server.close();
    });
}
