var fs = require('fs'),
    path = require('path'),
    _ = require('underscore'),
    glob = require('glob');

exports.createDefault = function() {
    var defaultConfig = {
        srcDir: "src",
        testDir: "test",
        reportsDir: "./reports",
        sources: [],
        tests: ["*Spec.js"]
    };

    return JSON.stringify(defaultConfig, null, '\t');
};

exports.createConfig = function(argv) {
    var filepath = argv.c;
    var conf = parseJSON(filepath);

    if (conf.srcDir) {
        conf.srcDir = absolutizeDir(filepath, conf.srcDir);
    }

    conf.testDir = absolutizeDir(filepath, conf.testDir);
    conf.reportsDir = absolutizeDir(filepath, conf.reportsDir);
    conf.tests = expandTests(conf.testDir, conf.tests);

    // command line opts take precedence over conf opts
    _.extend(conf, argv);

    conf.staticPort = argv.p;
    conf.staticUrl = 'http://localhost:' + conf.staticPort + '/';

    conf.runner = argv.r || conf.runner || 'zombie';
    conf.browser = argv.b || conf.browser || 'firefox';
    conf.testType = conf.testType || 'jasmine';

    return conf;
};

function parseJSON(filepath) {
    var data = fs.readFileSync(filepath, 'utf-8'),
        jsonData = {};
    
    try {
        jsonData = JSON.parse(data);
    } catch (e) {
        throw new Error('Error parsing json in config file: ' + e);
    }
    
    return jsonData;
}

function absolutizeDir(root, path) {
    // append on the trailing slash if it isn't there already
    if (path.substr(-1) !== '/') {
        path = path + '/';
    }
    
    if (path.charAt(0) === '/') {
        return path;
    }
    
    var absPath = fs.realpathSync(root);
    absPath = absPath.substring(0, absPath.lastIndexOf('/')); // chop off filename
    
    return absPath + '/' + path;
}

function expandTests(absPath, testFiles) {
    var expanded = [],
        relPathStart = absPath.length,
        i;

    // expand out all the testFile paths on the file system
    for (i = 0; i < testFiles.length; i++) {
        expanded = expanded.concat(glob.sync(absPath + testFiles[i]));
    }

    // remove the absPath so the expanded tests work as url paths,
    // we do the basename/absPath dance due to some odd behaviors
    // with cwd in the glob module
    for (i = 0; i < expanded.length; i++) {
        var fullPath = absPath + path.basename(expanded[i]);
        expanded[i] = fullPath.substring(relPathStart);
    }
    
    return expanded;
}