var fs = require('fs'),
    path = require('path'),
    connect = require('connect'),
    handlebars = require('handlebars'),
    ZombieRunner = require('./runners/zombie_runner').ZombieRunner,
    SeleniumRunner = require('./runners/selenium_runner').SeleniumRunner,
    PhantomJsRunner = require('./runners/phantomjs_runner').PhantomJsRunner;

exports.startServer = function(conf) {
    var app = connect();
    app.use(connect.static(path.resolve(__dirname, '..', conf.testType)));
    app.use(connect.static(conf.testDir));

    if (conf.srcDir) {
        app.use(connect.static(conf.srcDir));
    }

    generateSpecRunner(conf);

    return app.listen(conf.staticPort);
};

exports.createRunner = function(conf) {
    var runnerType = conf.runner,
        serverUrl = conf.staticUrl,
        runner;
    
    switch(runnerType.toLowerCase()) {
        case 'zombie': runner = new ZombieRunner(conf); break;
        case 'selenium': runner = new SeleniumRunner(conf); break;
        case 'phantomjs': runner = new PhantomJsRunner(conf); break;
    }

    return runner;
};

function generateSpecRunner(conf) {
    handlebars.registerHelper('script', function(context) {
        var scripts = [], i;

        for (i = 0; i < context.length; i++) {
            scripts.push('<script src="' + context[i] + '"></script>\n');
        }

        return scripts.join('');
    });

    var context = {
        sources: conf.sources,
        tests: conf.tests
    };

    var tmplSrc = path.join(__dirname, '..', conf.testType, 'specrunner.tmpl');

    if (conf.requirejs) {
        tmplSrc = path.join(__dirname, '..', conf.testType, 'require-specrunner.tmpl');
        context.requireConfig = new handlebars.SafeString(getRequireConfig(conf));
        context.requireSpecs = new handlebars.SafeString("['" + conf.tests.join("','") + "']");
    }

    var html = fs.readFileSync(tmplSrc, 'utf-8');
    var tmpl = handlebars.compile(html);

    fs.writeFileSync(path.join(__dirname, '..', conf.testType, 'specrunner.html'), tmpl(context), 'utf-8');
}

function getRequireConfig(conf) {
    var filepath = path.join(conf.srcDir, conf.requirejs);
    var text = fs.readFileSync(filepath, 'utf-8');

    var startPos = text.indexOf('require.config(');

    if (startPos === -1) { return ''; }

    var endPos = text.indexOf('});', startPos);

    return text.slice(startPos, endPos+3);
}
