var fs = require('fs');

var Logger = function(conf) {
    var reportsDir = conf.reportsDir;
    
    // Create the reports dir if it doesn't exist
    
    try {
        fs.statSync(reportsDir);
    } catch (e) {
        fs.mkdirSync(reportsDir, 0755);
    }
    
    this.conf = conf;
    
    this.totals = {
        passed:  0,
        failed: 0,
        count: 0
    };
};

Logger.prototype = {
    writeResults: function(reporter) {
        var conf = this.conf,
            reports = reporter.reports,
            report,
            results,
            failures,
            filename, 
            i, j;
            
        for (i = 0; i < reports.length; i++) {
            report = reports[i];
            results = report.results;
            failures = report.failures;
            filename = conf.reportsDir + "/" + report.filename;
            
            this.totals.passed += results.passed;
            this.totals.failed += results.failed;
            this.totals.count += results.total;
            
            if (results.failed > 0) {
                console.log('');
                console.log(report.name + ": " +  results.failed + " of " + results.total + " failed.");

                for (j = 0; j < failures.length; j++) {
                    console.log('');
                    console.log('\t' + failures[j].spec);
                    console.log('\t' + failures[j].trace);
                }
            } else {
                console.log(report.name + ": " +  results.passed + " of " + results.total + " passed.");
            }
            
            fs.writeFile(filename, report.text, 'utf-8', function(err) {
                if (err) {
                    console.log("Error writing to: " + filename);
                }
            })
        }
    },
    
    getTotals: function() {
        var totals = this.totals;
        
        return "\nFinal Summary: \n" + totals.passed + " passed, " + totals.failed + " failed, " + totals.count + " total.";
    }
};

exports.Logger = Logger;
