var soda = require('soda'),
    Logger = require('../logger').Logger;

function SeleniumRunner(conf) {
    this.conf = conf;
    this.browser = soda.createClient({
        host: 'localhost'
        , port: 4444
        , url: conf.staticUrl
        , browser: conf.browser
    });

    // Uncomment the lines below to enable command logging
    // this.browser.on('command', function(cmd, args){
    //     console.log(' \x1b[33m%s\x1b[0m: %s', cmd, args.join(', '));
    // });
}

SeleniumRunner.prototype = {
    
    /**
     * Starts the runner and executes all the test pages asynchronously. After the last test
     * is run, if a callback was provided, it will executed. This is useful for cleaning up
     * resources after the test run.
     *
     * @method start
     * @param callback {Function} an optional callback function
     */
    start: function(callback) {
        var self = this,
            conf = self.conf,
            browser = self.browser,
            logger = new Logger(conf);
        
        callback = callback || function() {};

        function specsComplete(reporter) {
            logger.writeResults(reporter);
            console.log(logger.getTotals());
            
            browser.close(function(err) {
                browser.testComplete(function(err) {
                    if (err) {
                        console.log("Error marking selenium session as complete: " + err);
                    }
                });
            });

            callback();
        }

        function getResults() {
            browser.getEval("window.JSON.stringify(window.reporter);", function(err, body, res) {
                var reporter;
                
                try {
                    reporter = JSON.parse(body);
                    
                    if (!reporter.finished) {
                        return setTimeout(getResults, 500);
                    }
                } catch (e) {
                    console.log(e);
                }
                
                specsComplete(reporter);
            });
        }
        
        browser.session(function(err) {
            browser.open('specrunner.html', function(err, body, res) {
                getResults();
            }); 
        });
    }
};

exports.SeleniumRunner = SeleniumRunner;
