var Browser = require('zombie').Browser,
    Logger = require('../logger').Logger;

/**
 * A headless test runner that loads test pages and outputs the test results to file.
 *
 * @class Runner
 * @constructor
 * @param filepath {String} the path to the json config file, can be relative or absolute
 */
function ZombieRunner(conf) {
    this.conf = conf;
}

ZombieRunner.prototype = {
    
    /**
     * Starts the runner and executes all the test pages asynchronously. After the last test
     * is run, if a callback was provided, it will executed. This is useful for cleaning up
     * resources after the test run.
     *
     * @method start
     * @param callback {Function} an optional callback function
     */
    start: function(callback) {
        var self = this,
            conf = self.conf,
            logger = new Logger(conf),
            browser = new Browser();

        callback = callback || function() {};

        browser.visit(conf.staticUrl + 'specrunner.html', function() {
            if (browser.errors && browser.errors.length > 0) {
                console.log(browser.errors);
                callback();
                return;
            }

            function specsComplete(window) {
                return window.reporter.finished;
            }

            browser.wait(specsComplete, function() {
                var reporter = browser.window.reporter;
            
                logger.writeResults(reporter);
                console.log(logger.getTotals());
                callback();
            });
        });
    }
};

exports.ZombieRunner = ZombieRunner;
