(function() {    
    function ISODateString(d) {
        function pad(n) { return n < 10 ? '0'+n : n; }
        
        return d.getFullYear() + '-'
            + pad(d.getMonth()+1) +'-'
            + pad(d.getDate()) + 'T'
            + pad(d.getHours()) + ':'
            + pad(d.getMinutes()) + ':'
            + pad(d.getSeconds());
    }

    QUnitReporter = function() {
        var self = this,
            text,
            report,
            test;

        self.started = false;
        self.finished = false;
        self.reports = [];

        QUnit.log(function(data) {
            if (! data.result) {
                var failure = {
                    spec: test.name,
                    trace: 'Expected ' + data.expected + ', got ' + data.actual + '. ' + data.message + ' (' + data.source + ')'
                };

                test.failures.push(failure);
                report.failures.push(failure);
            }
        });

        QUnit.testStart(function(data) {
            test = {
                name: data.name,
                failures: []
            };
        });

        QUnit.testDone(function(data) {
            report.tests.push(test);
        });

        QUnit.moduleStart(function(data) {
            text = ['<?xml version="1.0" encoding="UTF-8" ?>\n'];

            report = {
                name: data.name,
                filename: 'TEST-' + data.name.replace(/\s/g, '') + '.xml',
                results: {},
                tests: [],
                failures: [],
                started: new Date()
            };
        });

        QUnit.moduleDone(function(data) {
            report.finished = new Date();
            report.results.passed = data.passed;
            report.results.failed = data.failed;
            report.results.total = data.total;

            var elapsed = (report.finished - report.started)/1000,
                testResult,
                i, j;

            text.push('<testsuite name="' + report.name + '" errors="0" failures="' + data.failed 
                + '" tests="' + data.total + '" time="' + elapsed + '" timestamp="' + ISODateString(new Date()) + '">\n');

            for (i = 0; i < report.tests.length; i++) {
                testResult = report.tests[i];

                if (testResult.failures.length > 0) {
                    text.push('\t<testcase classname="'+ report.name + '" name="' + testResult.name + '" time="0">\n');

                    for (j = 0; j < testResult.failures.length; j++) {
                        text.push('\t\t<failure>' + testResult.failures[j].trace + '</failure>\n');
                    }

                    text.push('\t<testcase/>\n');
                } else {
                    text.push('\t<testcase classname="'+ report.name + '" name="' + testResult.name + '" time="0" />\n');
                }

            }

            text.push('</testsuite>\n');

            report.text = text.join('');

            self.reports.push(report);
        });

        QUnit.begin(function() {
            self.started = true;
        });

        QUnit.done(function(data) {
            self.finished = true;
        });
    };
})();